


function []=structuralSegmentation_PACK(inputWAV,outputLAB)
% STRUCTURAL SEGMENTATION Music structure inference algorithm\\based on
% symbolic data analysis
%
% structuralSegmentation(inputWAV,outputLAB)
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs : inputWAV : path to input audio wav file (includes filename)
%          outputLAB: path to output structure text file (includes
%          filename, can be a *.lab as well as a *.txt file)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright 2010 Gabriel SARGENT (Universite de Rennes1, IRISA)
% This software is distributed under the terms of the GNU Public License
% version 3 (http://www.gnu.org/licenses/gpl.txt)
% If you find it useful, please cite the following references:
% G. Sargent, F. Bimbot and E. Vincent,
%``A regularity-constrained Viterbi algorithm and its application to the structural segmentation of songs''
% Proceedings of the International Symposium on Music Information Retrieval, 2011.
% G. Sargent, S.A. Raczynski, F. Bimbot, E. Vincent and S. Sagayama,
%``A music structure inference algorithm based on symbolic data analysis (Extended abstract) ''
% MIREX, 2011.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% inputWAV='/Users/gsargent/Work/Packs/Corpus/RWC_audio_wav/Popular_section/4.wav';
% outputLAB='./4.txt';

alpha=[0.5];
lrun=[1];
allLambda=[0.15];
tau=16;

%load('./psPropsCell.mat');%learned weights associated to structural period sets (RWC pop)
%params=psPropsCell{1,1};
%nbmaxPS=params(1);
%blockSizeRange=params(2);
%psTab=psPropsCell{2,1};
%number of classes/centroids for VQ :
%nC=[8,12,16,24,32];
%lnC=length(nC);


% Filename :
indSlash=strfind(inputWAV,'/');%filename localization in the path (after the last "/") :
fileName=inputWAV(indSlash(end)+1:end-4);
disp(['Processing ' inputWAV(indSlash(end)+1:end)]);
% FEATURE EXTRACTION :
%%%%%%%%%%%%%%%%%%%%%%
featDir='./features';

fprintf('Step 1 : feature extraction...\n');
% Beats and downbeats : calling script by Davies  :
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[beatFile]=callingDownbeatExtractor(inputWAV,featDir);

%Chord estimation file : calling software by Ueda  :
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[chordFile]=callingChordEstimator(inputWAV,featDir);

%using already-extracted features
%beatFile name :
indSlash=strfind(inputWAV,'/');%filename localization in the path (after the last "/") :
beatFile=[inputWAV(indSlash(end)+1:end-4) '_BEATS.TXT'];
%chordFile name :
indSlash=strfind(inputWAV,'/');%filename localization in the path (after the last "/") :
chordFile=[inputWAV(indSlash(end)+1:end-4) '_CHORDS.rec'];
fprintf(' \nFeature extraction done.\n');





fprintf('Step 2 : structural segmentation...')
% SNAP GENERATION      :
%%%%%%%%%%%%%%%%%%%%%%%%
%reading beats
%beatFile=strcat('RM-P',songNumber1,'.BEAT_ESTIM.TXT');    %filename
quarternotes=textread([featDir '/' beatFile]);     %quarternotes (="noires")
quarternotesInd=[384, 48, 96, 144];
indBeats=find(quarternotes(:,3)==quarternotesInd(1));%localizing first quarternote of the measure %beats=quarternotes(:,1)*10^(-2);%(indBeats,1)*10^(-2); %beats in seconds

%checking that the beats chosen are well separated by 3 values (quarter beats are used by AIST)
testperiodic=indBeats(2:end)-indBeats(1:end-1);%number of timer in the measures
if(sum(testperiodic-4)~=0)
    fprintf(['FYI, there are some changes in metrics in ' fileName '\n']);%variations sur la p?riodicit? des labels des beats
    indBeats21=indBeats(testperiodic==4)+2;%localising 3rd time in every measure of 4 times (not the other measures)
    indBeats2=sort([indBeats; indBeats21],'ascend');

else
    indBeats21=find(quarternotes(:,3)==quarternotesInd(3));%localising third quarternotes of the measures in the case where there are only measures of 4 times.
    indBeats2=sort([indBeats ;indBeats21],'ascend');

end;
avQuarterNoteDuration=mean((quarternotes(2:end,1)-quarternotes(1:end-1,1))*10^(-2)); %average duration of a quarternote
%vectavDuration(f)=avQuarterNoteDuration;

%choosing a higher/lower beat scale according to the value of the average quarternote duration in order to get the "snap" scale.     
avDuration2=mean((quarternotes(indBeats2(2:end),1)-quarternotes(indBeats2(1:end-1),1))*10^(-2)); %average duration of a beat, at a less precise scale
avDuration1=mean((quarternotes(indBeats(2:end),1)-quarternotes(indBeats(1:end-1),1))*10^(-2)); %average duration of a beat, at a less precise scale
[avDurMin,bScale]=min(abs([avQuarterNoteDuration,avDuration2,avDuration1]-1));    
switch(bScale)  %for RWC POP, the duration of the quarter notes are contained in the interval [0.3s, 0.9s].
    case 1
        beats=quarternotes(:,1)*10^(-2);%beats in seconds
        nbBeats=length(beats);
    case 2
        beats=quarternotes(indBeats2,1)*10^(-2); %beats in seconds
        nbBeats=length(beats);
    case 3
        beats=quarternotes(indBeats,1)*10^(-2); %beats in seconds
        nbBeats=length(beats);            
end;
%avBeatDuration=mean(beats(2:end,1)-beats(1:end-1,1));
 

% READING CHORDS ESTIMATIONS :
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Chord and Key estimations symbols :

chordKeyFile=[featDir '/' chordFile];
[chordSequence,keySequence,nbClassChord,nbClassKey]=readChordKeyEstim(chordKeyFile,beats);

%referencing the useful symbol indices and mapping them to
%1:nbUsefulSymbols :
%seq=chordSequence;nbClass=71;
[mappedChordSequence,mapTab]=referAndMap(chordSequence);


%Storing all useful sequences :
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
allSeq=mappedChordSequence;
seqNbClass=[max(mappedChordSequence)];

%Loop on symbolic features :
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[lseq,nbSymbFeats]=size(allSeq);

for iSeq=1%;[4,9,14]%nbSymbFeats]%1:nbSymbFeats

    %Symbolic feature selection
    seq=allSeq(:,iSeq);
    nbClass=seqNbClass(iSeq);

    %Similarity matrix calculation :
    [matSim]=matSimSymb(seq); 
    %figure; imagesc(matSim);colormap(1-gray);%pause;
    %Loop on the minimal run size filtering

    %lrun=8;%[4,8,16,32];
     for r=1:length(lrun);
        
        %filtering : only runs of size lrun(r) are kept
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        if(lrun(r)>1)
            [mat]=matRunSelection(matSim,lrun(r));
        else
            mat=matSim;
        end;

        for l=1:length(allLambda)


            %Viterbi with regularity constraints
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            lambda=allLambda(l);
            [segt]=bestPath(mat,lambda,tau,alpha);
            

            %last border "nbBeats+1" is set to the last beat of the song 
            segt(segt>nbBeats)=nbBeats;


            segtDisp=zeros(1,length(mat));
            segtDisp(segt)=1;
            
            
        end;

       
    end

end;
fprintf(' Done.\n');

% Clustering step using the Automaton criterion:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('Step 3 : structural segments clustering...');
%Hierarchical fusion of automaton branches  :
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%segmentation format : adding [#] segments (not taken into
%account in the final annotation)

segSize=segt(2:end)-segt(1:end-1);
bigBlocksLabels0=num2str(segSize');
[N,strLen]=size(bigBlocksLabels0);
%shifting the spaces to the right (input format for branchFacto function, comes from annotations from a former study)

for i=1:N
    spaceTest=strfind(bigBlocksLabels0(i,1),' ');
    if (~isempty(spaceTest) && spaceTest(1)==1)
        bigBlocksLabels0(i,:)=[bigBlocksLabels0(i,spaceTest(end)+1:strLen), char(32*ones(1,spaceTest(end)))];
    end;
end;    

songBound=char(ones(1,strLen)*32);%3 spaces
songBound(1)='#';
bigBlocksLabels0=[songBound;bigBlocksLabels0;songBound];

segBoundaries0=[0,segt,segt(end)];

%case where all blocks have different lengths, prefixes and suffixes are supported :
[pB, pS, usefulRegTab, clusterTab]=branchFacto(allSeq, segBoundaries0, seqNbClass, bigBlocksLabels0);


%"y-logP" criterion :
%%%%%%%%%%%%%%%%%%%%%
logP=log(pS)+log(pB);
%locating the absciss of the minimal value of -log(aP)+y = the maximal value of log(aP)-y :
%Calcul des paramtres de la droite passant par les extrmits de log(aP)
coefDir=(logP(end)-logP(1))/(length(logP)-1);
ordOrigin=logP(1)-coefDir*1;

y=[1:length(logP)]'*coefDir+ordOrigin;
%locating the absciss of the minimum value of -log(aP) = the maximum value of log(aP)
crit1=logP-y; 

[valeurMax,iMax]=max(crit1);            

%N=nbSeg-2;
%selection of the part of clusterTab associated to "indiceMax" groupings
ind0=N*(iMax-1)+(1:N);
clusterTab0=clusterTab(ind0,1:N);
%version without any empty cluster (the first element of each line has
%to differ from zero)
clusterTab1=[];
for i=1:N
    if(clusterTab0(i,1)~=0)
        clusterTab1=[clusterTab1;clusterTab0(i,:)];
    end;
end;
%Vector which contains the similarity labels (integers)
segLab=zeros(N,1);
%loop on the segments' number
for k=(1:N)+1
    k;
    [x,y]=find(k==clusterTab1);
    segLab(k-1)=x;
end;
fprintf(' Done.\n');

%Saving the obtained segmentation (every label =1)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% ! check if beginning of segmentation taken into account
segt(segt>length(beats))=length(beats);
segmentation=[beats(segt(1:end-1)),beats(segt(2:end))];
segSize=segt(2:end)-segt(1:end-1);

%With label = segment size in snaps
% fid = fopen(outputLAB,'w');
% for k=1:size(segmentation,1)
%     fprintf(fid,'%.7f\t%.7f\t%d\n',segmentation(k,1:2),segSize(k));
% end;
% fclose(fid);


%With label = similarity
symbols=char([65:90 97:122 48:57 36 64]);
fid = fopen(outputLAB,'w');
for k=1:size(segmentation,1)
    fprintf(fid,'%.7f\t%.7f\t%s\n',segmentation(k,1:2),symbols(segLab(k)));
end;
fclose(fid);
fprintf([inputWAV(indSlash(end)+1:end) ' processed.\n\n'])
            
return;



function [usefulRegTab, clusterTab]=allClusterStates(regTab,clustName)

% ALLCLUSTERSTATES function producing the state of each clusters during the hierarchical
% clustering
%
% [usefulRegTab, clusterTab]=allClusterStates(regTab,clustName)
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs : - regTab       = 2-column table containing the ordered dyadic groupings of the clustering 
%                           (each line contains the number of the 2 clusters to gather
%                            the 1st line is the first grouping,etc...)
%          - clustName    = label of the useful clusters (numbers)
% Outputs: - usefulRegTab = 2-column table containing only the ordered USEFUL dyadic
%                           groupings of the hierarchical clustering, regards to the
%                           numbers in 'clustname'
%          - clusterTab   = table which references the elements of every
%                           cluster for each step of the hierarchical clustering, whose
%                           order is described in 'usefulRegTab'
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Number of elements to cluster (segments in our case) :
nbSeg=length(clustName);

clusterTab=zeros(nbSeg*nbSeg, nbSeg+1);     % "+1" : column containing the number of elements of each cluster
useful=zeros(length(regTab),1);               %useful lines of regTab are noted "1", "0" if not.


%Initialization : (we're ignoring cluster "1" in our case - silent segment)
clusterTab(1:nbSeg,1)=clustName;
clusterTab(1:nbSeg,nbSeg+1)=ones(nbSeg,1);

%Iterations :
i=1;
for k=1:length(regTab)
    
    
    %i-th line of regTab is considered.
    %we'd like to find if these 2 elements are already on the same line,
    %considering the previous step of hierarchical clustering.
    
    [x1,y1]=find(clusterTab(1+nbSeg*(i-1):nbSeg*i,1:nbSeg)==regTab(k,1));
    [x2,y2]=find(clusterTab(1+nbSeg*(i-1):nbSeg*i,1:nbSeg)==regTab(k,2));
    
    
    if(x1-x2~=0)
        
        useful(k)=1;
        
        %first, duplicate the states of the clusters from the previous step
        clusterTab(1+nbSeg*i:nbSeg*(i+1),:)=clusterTab(1+nbSeg*(i-1):nbSeg*i,:);
        
        nbEltsx1=clusterTab(x1+nbSeg*(i-1),nbSeg+1);
        nbEltsx2=clusterTab(x2+nbSeg*(i-1),nbSeg+1);

        %gathering the elements of the 2 lines x1 and x2 from old clusters
        %state to the new one, line x1.
        clusterTab(x1+nbSeg*i,1:nbEltsx1+nbEltsx2)=[clusterTab(x1+nbSeg*(i-1),1:nbEltsx1) clusterTab(x2+nbSeg*(i-1),1:nbEltsx2)];
        %updating the number of elements of modified clusters
        clusterTab(x1+nbSeg*i,nbSeg+1)=nbEltsx1+nbEltsx2;        
        clusterTab(x2+nbSeg*i,1:nbSeg+1)=zeros(1,nbSeg+1);  %no elements inside this line anymore
        
        i=i+1;
        
    end;
end;
[indU]=find(useful==1);
usefulRegTab=regTab(indU,:);

%[regTab usefulRegTab]
return;


function [seg]=bestPath(mat,lambda,tau,alpha)

% BESTPATH viterbi function
%
% [seg,thetaSeg]=bestPath(mat,lambda,tau,alpha,nbmaxPS,blockSizeRange,psTab)
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs : - mat       = binary similarity matrix in snaps
%          - lambda    = balance factor between content-based cost and
%                        regularity cost.
%          - tau       = structural pulse period (in snaps)
%          - alpha     = regularity cost convexity factor (power of the
%                        regularity function in regCost)
%
% Outputs: - seg       = structural segment borders in snaps 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

N=length(mat);
H=3*tau;%maximal length of historic window
infinity=100;

C=zeros(1,N+1);%cumulated cost matrix %C(1) is volontarly initialized to zero
PHI=zeros(1,N+1);% cumulated phi associated to cumulated cost
PSI=zeros(1,N+1);% cumulated psi associated to cumulated cost

gamma=zeros(H,N+1);%elementary transition cost matrix %gamma(h,t) is transition cost from instant t-h to instant t.
phi0=zeros(H,N+1);
psi0=zeros(H,N+1);

optTheta=zeros(H,N+1);%antecedent localization in lag
b=zeros(1,N+1);   %can be noted "optimal h for instant t" -> antecedent localization in time
b(1)=1;

for t=2:N+1
    %t
    H1=min(H,t-1); %taking edge effect into account(verif with FB)
    
    for h=1:H1  %loop on historic window length h
        
        %calculation of regularity cost "psi" (linked to structural period)
        psi=regCost(h,tau,3,alpha);%,psPropsCell);
        
        %theta=index of other other parts of the signal to be compared to the
        %reference window [t-h:t-1] (window lengths must be the same).
        
        %calculation of similarity cost "phi"
        phi=infinity*ones(N,1);
        for theta=[h:(t-1)-tau,(t-1)+tau:N]  %theta indexes the last point of the windows compared to the reference window [t-h:t-1]
            
            % selection of the part of the input matrix's sub-diagonal referring to the
            % frame-by-frame comparison of the reference and compared
            % windows
            simVect=zeros(1,h);
            for p=1:h
                simVect(1,h-p+1)=mat(theta+1-p,t-p);  %keeping chronological order
            end;
            
            %phi function : summed similarity
            phi(theta)=sum(1-simVect);
            
        end;
        
        %smallest cost and theta with smallest cost, considering instant t
        %and window size h
        [phih,optTheta0]=min(phi);
        phih=phih+0.0001;
        optTheta(h,t)=optTheta0-h+1;
        %elementary transition cost from instant t-h to instant t
        %gamma(h,t)=phih+lambda*tau*psi;
		gamma(h,t)=(phih+lambda*tau*psi)/(1+lambda*tau);%Norma2
        %gamma(h,t)=(1-lambda)*phih+lambda*psi;%norma1
		phi0(h,t)=phih;
        psi0(h,t)=psi;
        
    end;
    
    
    %selection of the best antecedent for instant t, according to cumulated
    %cost
    cost=zeros(1,H1);
    for h=1:H1
        cost(h)=C(t-h)+gamma(h,t);
    end;
    
    [C(t),b(t)]=min(cost);
    
    PHI(t)=phi0(b(t),t);
    PSI(t)=psi0(b(t),t);
    
end;

%Backtracking

t=N+1;%absolute time
seg=t;
thetaSeg=1;%arbitrarily chosen %optTheta(b(t),t);

PHItot=PHI(t);
PSItot=PSI(t);

while(t>1)
    
    thetaSeg=[optTheta(b(t),t),thetaSeg];
    t=t-b(t);
    seg=[t,seg];
    PHItot=PHItot+PHI(t);
    PSItot=PSItot+PSI(t);
    
end;
%seg(1:end-1);
thetaSeg;%useful for printing the best path on the similarity matrix (seg=absciss, thetaseg=ordinates)

return;



function [segCompo]=blockLabConvert(bigBlocksLabels,seglen)
% BLOCKLABCONVERT function converting each segment size labels (string) into
% a set of integers : size of radical, size of prefix, size of suffix.
%
%function converting the autonomous blocks labels (strings) into a tab
%containing the length of :
%radicals(segCompo(:,1))
%prefixes(segCompo(:,2))
%suffixes(segCompo(:,3))
%
% [seg,thetaSeg]=bestPath(mat,lambda,tau,alpha,nbmaxPS,blockSizeRange,psTab)
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs : - bigBlocksLabels = vector of strings (one column)
%          - seglen    = vector containing the number of detected snaps for
%                        each segment.
%
% Outputs: - segCompo  = 3-column table containing respectively the size of
%                        the stem (1st column),prefix (2nd col.), suffix (3rd col.)
%                        for each segment (1segment = 1 line)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

nbSeg=length(bigBlocksLabels);
%assumption 1 : prefixes and suffixes are always shorter than the related radical
%assumption 2 : there is a maximum of 2 numbers in each label
%assumption 3 : all label containing an x is given its number of snaps
%               determined by the algorithm (other possible solution : radical length set directly to 0??)

%! (bad approximation) : all "&" encountered are treated the same way than
%"+"

segCompo=zeros(nbSeg,3);
infix=zeros(nbSeg,1);

for n=2:nbSeg-1 %ignoring the # extreme labels
    
    label=bigBlocksLabels(n,:);
    indx=strfind(label,'x');
    if(length(indx)==0)%if the current segment contains an "x" then consider it as a irregular block with length its number of snaps counted by the algorithm

        %Label adjustments :
        
        %ignoring the spaces at the end of the label if present
        indSpaces=strfind(label,' ');
        
        if(~isempty(indSpaces))
            label=label(1:indSpaces(1)-1);
        end;

        %standardize the label format : if the label doesn't begin with '-',
        %then insert a ' ' at the beginning
        if(label(1)~='-')
            label=[' ' label];
        end;
        
        lenlab=length(label);
        %Localizing operators (to locate the numbers in the next step)
        [indplus] =find(label=='+');
        [indminus]=find(label=='-');
        [indand]  =find(label=='&');
        [indspace]=find(label==' ');%useful to locate the 1st number ONLY
        
        
        %vector containing all operator indices and boundaries of numbers at the extremes of 'label' :
        indAll=[sort([indplus indminus indand indspace]) (lenlab+1)];
        nbLabNum=length(indAll)-1;
        convertedLengths=zeros(1,nbLabNum-length(indand));%infixes are treated separately = put in another vector named 'infix"
        
        %Localizing and converting the numbers contained in 'label'
        k=0;
        for i=1:nbLabNum
            num=str2double(label(indAll(i)+1:indAll(i+1)-1));

            if(label(indAll(i))=='-')
                num=num*(-1);
                k=k+1;
                convertedLengths(k)=num;
            end;
            
            if(label(indAll(i))=='&')
                infix(n)=num;
            else
                k=k+1;
                convertedLengths(k)=num;                
            end;

        end;
        
        %Storing the values in segCompo
        segCompo(n,1)=max(convertedLengths);
        if(nbLabNum>1)
            [valmin,indmin]=min(convertedLengths);
            segCompo(n,indmin+1)=valmin;%prefix=> 2nd column, suffix=> 3rd column
        end;      
            
    else
        
        segCompo(n,1)=seglen(n);
    end;
end;


%considering infixes as suffixes
segCompo(:,3)=segCompo(:,3)+infix(:);
return;

function [pB0,pS0,usefulRegTab,clusterTab]=branchFacto(seq, bounds, nC, bigBlocksLabels)
% BRANCHFACTO clustering function, by automata branch factorization
%
% [seg,thetaSeg]=bestPath(mat,lambda,tau,alpha,nbmaxPS,blockSizeRange,psTab)
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs : - seq      = sequence of symbols describing a whole music piece at the snap scale
%                     (=one line of length L);seq(i) contains the class number of the related feature
%                     => symbols as numbers
%          - bounds   = segment boudaries in snaps (bounds(i) has to be the index of the
%                     first symbol associated to the ith segment of the current sequence
%                     seq); has to contain the indexes of the 2 extreme
%                     segments noted [#] (usually, silent segment);
%          - nC       = size of the alphabet of symbolic features used
%          - bigBlocksLabels = vector of strings gathering the size of the structural segments in snaps
%        
%
% Outputs: - pB0           = block probability vector : pB0(i) is the probability for ith automaton (with N-i+1 branches) 
%                            to choose the good sequence of segment classes (or automaton branches) to find the feature 
%                            sequence 'seq'; 1 line = 1 clustering step (initial step = line 1), 1 column = 1 structural segment.
%          - pS0           = symbol probability vector : pS0(i) is the probability for ith automaton (with N-i+1 branches) 
%                            to choose the good sequence of symbols in a particular branch of the automaton.
%          - usefulRegTab = 2-column table containing only the ordered USEFUL dyadic
%                           groupings of the hierarchical clustering, regards to the
%                           numbers in 'clustname'
%          - clusterTab   = table which references the elements of every
%                           cluster for each step of the hierarchical clustering, whose
%                           order is described in 'usefulRegTab'
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
L=length(seq);
nbSeg=length(bounds)-1;  
bounds1=bounds;     %=1 column
alphabet=char([65:90 97:122 48:57 36 64 123 125 40 41 91 93 95]);   %71symbols



%sequence of symbols associated to the sequence of class numbers seq
seqSymb=alphabet(seq);

%length of every segment
seglen=bounds1(2:end)-bounds1(1:end-1);

%each label describing the size of a segment is "converted" as a set of 3
%numbers : [radical size, prefix size, suffix size]:
[segCompo]=blockLabConvert(bigBlocksLabels,seglen);

segCompo(:,2:3)=ceil(segCompo(:,2:3));

indNegvals=find(segCompo<0);
segCompo1=segCompo;
segCompo1(indNegvals)=0;
lmax=sum(max(abs(segCompo1),[],1));     %length of the standard 3D matrix describing a block in matrix T (= longest radical + longest prefix + longest suffix, truncations (negative values)are not considered, and have their value set to 0)

%detecting useful segments (non-empty segments, which contain at least 1 snap)
usefulSeg=find(seglen(2:end-1)~=0)+1;   %excluding systematically the "#" boundaries (includes a shift of 1)

nbUSeg=length(usefulSeg);%number of useful segments

%Distance matrix :
%%%%%%%%%%%%%%%%%%

%Matrix containing the distance (1st column) between the ith segment
%(number stored in the 2nd column) and the jth segment (number stored in
%the 3rd column)
mDist=zeros(sum([1:nbUSeg-1]),3);    %nbSeg-3 => we're not considering the first set of distances between the 1st silent segment and the other segments; neither the last set of distances between the last silent segment and other segments.
k=0;    %index of the number of distance calculations

%loop on the segments; by convention, the first and last segments are
%silences -> annotated '#', and are not taken into account in our
%calculations.

for n=1:nbUSeg
    
    n1=usefulSeg(n);
    
    s1=seqSymb(bounds1(n1):bounds1(n1+1)-1);
    
    for j=n+1:nbUSeg
        
        j1=usefulSeg(j);
        s2=seqSymb(bounds1(j1):bounds1(j1+1)-1);
        k=k+1;
        mDist(k,1)=EditDist(s1,s2,1,1,1)/max([seglen(n1),seglen(j1)]);
        mDist(k,2)=n1;
        mDist(k,3)=j1;
        
    end;
    
end;

%Hierarchical clustering :
%%%%%%%%%%%%%%%%%%%%%%%%%%
[mDist(:,1),indAscend]=sort(mDist(:,1),'ascend');%ordering the normalized edit distances between segments
mDist(:,2)=mDist(indAscend,2);
mDist(:,3)=mDist(indAscend,3);

%Matrix containing clusters for each step of hierarchical clustering +
%useful groupings from "mDist(:,2:3)".
[usefulRegTab, clusterTab]=allClusterStates(mDist(:,2:3),usefulSeg);

%Calculating the modeling score related to each step of the hierarchical clustering :
maxRegNb=length(usefulRegTab);%maximal number of groupings

%Creating and fulfilling the transition matrix T :
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


T=zeros((nC+1)*nC,nC+1,nbUSeg*(lmax+1));          %(+1) -> additional "input" and "output" states; every "useful" block has a 3D matrix of length the maximal block length +1, considering the output state transition.
indSegT=ones(1,nbUSeg)+(0:nbUSeg-1)*(lmax+1); %indexes of the beginning of the blocks in the matrix T
seglenT=zeros(nbUSeg,nbSeg);                  %matrix containing for each clustering step (each line) the size of the branch linked to each block (each column).
seglenT(1,1:nbSeg)=seglen;

%length of the biggest prefix :
pref=segCompo(:,2); %pref(i)=length of the refix associated to the i-th block, rounded to its upper integer in case of decimal length
pref=ceil(pref);
prefMax=max(pref);  %value of the largest prefix encountered in the current song


pB=zeros(nbUSeg,nbSeg);                       
pS=zeros(nbUSeg,nbSeg);                       

%initial step : Transision of the nbSeg branches (segments)
for n=1:nbUSeg
    
    n1=usefulSeg(n);
    seg=seq(bounds1(n1):bounds1(n1+1)-1);% sequence of symbol numbers associated to segment i

    T(nC+1,nC+1,indSegT(n):indSegT(n)+prefMax-pref(n1)-1)=1;        %all transitions before the sequence states are  "'input state' to 'input state' transitions".
    T(seg(1),nC+1,indSegT(n)+prefMax-pref(n1))=1;                   %considering the transition from the input of the automaton to the first symbol of the drown branch ("ghost symbol in" to "1st symbol/state of the branch" -> last column of this matrix)
    for t=2:seglen(n1)
        
        t1=indSegT(n)+prefMax-pref(n1)+t-1;
        T(seg(t),seg(t-1),t1)=1;

    end
    T(nC+1,seg(seglen(n1)),indSegT(n)+prefMax-pref(n1)+seglen(n1)-1+1)=1;        %considering the transition between the last symbol of the drown branch and the "output state/symbol".
    
end

pB(1,usefulSeg(1:nbUSeg))=ones(1,nbUSeg)*(1/(nbUSeg));    %first, each bloc (segment, branch) can be drawn with the same probability (1st and last segments # are silent, and hav a probability of 0)
pS(1,usefulSeg(1:nbUSeg))=ones(1,nbUSeg);                 %At the initial step, if the drawn bloc is the good one, the good sequence of symbols is drawn with certitude(probability = 1).

%Iteration on the factorizations, according to hierarchical groupings
%already determined

for i=1:nbUSeg-1     %loop on the number of steps (1st segment is not grouped at the end of the clustering)
   
    %Number of non-empty clusters at the ith step :
    clustersi=clusterTab(1+nbUSeg*i:nbUSeg*(i+1),:);
    [xc,yc]=find(clustersi(:,1)~=0);
    nbClust=length(xc);
    
    
    for c=1:nbClust     %loop on the number of clusters at the ith step :
    
        c1=xc(c);                               %considered cluster 
        nbElts=clustersi(c1,nbUSeg+1);          %number of elements in teh cluster read in the last column of matrix 'clustersi'
        m=lmax+1;                               %+1 = also considering output transition.
        T1=zeros(nC+1,nC+1,m);                  %matrix which will contain the sum of transition matrices of segments associated to cluster c1 (not yet normalized)
        indEtapei=1+i*(nC+1):(i+1)*(nC+1);      %part of T associated to ith step.
        
        %sum of transition matrices of segments associated to cluster c1 
        pB0=0;
        for e=1:nbElts

            segUInd=find(usefulSeg==clustersi(c1,e));
            indseg=indSegT(segUInd):indSegT(segUInd)+lmax;    %(still considering output transition)
            
            T1(:,:,1:lmax+1)=T1(:,:,1:lmax+1)+T(1:nC+1,:,indseg);% transition matrices
            pB0=pB0+pB(1,clustersi(c1,e));                  %drawing probability of blocks (segments)
            
        end;
    
        %Normalisation
        for j=1:nC+1          %+1 => also considering the last column (input transition)
            
            for t=1:m
               
                somme=sum(T1(:,j,t));
                if(somme~=0)
                    T1(:,j,t)=T1(:,j,t)/somme;
                end;
                
            end
            
        end;
        
        %Updates
        for e=1:nbElts
            
            %Updating the part of T associated to ith step.
            segUInd=find(usefulSeg==clustersi(c1,e));
            indseg=indSegT(segUInd):indSegT(segUInd)+lmax;   %considering output transition
            T(indEtapei,:,indseg)=T1(:,:,1:lmax+1);
                        
            seg=seq(bounds1(clustersi(c1,e)):bounds1(clustersi(c1,e)+1)-1);
            
            %Probabilities on the drawing of "good" symbols of segments considered in cluster c1
            probas=zeros(1,seglen(clustersi(c1,e)));
            probas(1)=prod(T1(nC+1,nC+1,1:prefMax-pref(clustersi(c1,e))))*T1(seg(1),nC+1,prefMax-pref(clustersi(c1,e))+1);%produit des probas associes aux transitions in->in (1er terme, "in"=tat d'entre), ainsi que la transition "in" vers "1er symbole de la sous-squence" (deuxime terme)
            for t=1:seglen(clustersi(c1,e))-1
                probas(1,t+1)=T1(seg(t+1),seg(t),prefMax-pref(clustersi(c1,e))+t+1);
            end

            pS(i+1,clustersi(c1,e))=prod(probas)*T1(nC+1,seg(seglen(clustersi(c1,e))),prefMax-pref(clustersi(c1,e))+seglen(clustersi(c1,e))+1);%we take into account the probability associated to teh transition "last symbol of the sub-sequence (of a segment) to an "ou" state.
            %updating the probability of drawing blocs associated to c1 cluster
            pB(i+1,clustersi(c1,e))=pB0;
            
        end;

    end

end 

%Global probability for each step(factorisation)

P=prod([pB(:,usefulSeg(1:nbUSeg)) pS(:,usefulSeg(1:nbUSeg))],2);
pB0=prod(pB(:,usefulSeg(1:nbUSeg)),2);
pS0=prod(pS(:,usefulSeg(1:nbUSeg)),2);

%selecting the largest number of factorizations which has the highest value of P
[imax,junk]=find(P==max(P));
bestClusteringStep=imax(end)-1;% estimated nb of groupings "minus one" -> takes into account that the first value of P is associated to the initial clustering step (1 segment per cluster)

%associated clusters : reading the associated part in the clusterTab table.

clustersiBest=clusterTab(1+nbUSeg*bestClusteringStep:nbUSeg*(bestClusteringStep+1),:);
estimBigBlocksLabels=zeros(length(bounds)-1,1);


for n=1:nbUSeg
    [esstim,junk]=find(clustersiBest(:,1:nbUSeg-1)==usefulSeg(n));
    estimBigBlocksLabels(usefulSeg(n));
end;

AICpenalty=((nbUSeg-(1:nbUSeg)+1)*(1+nC+(nC*nC)*(lmax-1)))/(nC*nC); % division par nC*nC = normalisation arbitraire pour faciliter la recherhe de la pondration.

%Filling the segLengths table :
%%2 segments gathered in a same class have the same length = the length of
%%the longest one :
%evolution of the length of the segments according to the hierarchical
%clustering step in the following table :

segLengths(1,:)=seglen;
for q=1:nbUSeg-1
    
    segLengths(q+1,:)=segLengths(q,:);
    mV=max(segLengths(q,usefulRegTab(q,1)),segLengths(q,usefulRegTab(q,2)));
    segLengths(q+1,usefulRegTab(q,1))=mV;
    segLengths(q+1,usefulRegTab(q,2))=mV;
    
end;

return;

function [chordFileName]=callingChordEstimator(inputWAV,featDir)
% CALLINGCHORDESTIMATOR function which calls chord estimator by Ueda and saves resulting file in
%                       the 'features' folder
%
% [chordFileName]=callingChordEstimator(inputWAV,featDir)
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs : - inputWAV : path to input audio wav file (includes filename)
%          - featDir  : fpath to the directory in which extract teh
%                       features
%
% Outputs: - chordFileName : name of the file which contains chord
%                            estimations
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%featDir='./features';
cd('./chord');

%chordFile name :
indSlash=strfind(inputWAV,'/');%filename localization in the path (after the last "/") :
chordFileName=[inputWAV(indSlash(end)+1:end-4) '_CHORDS.rec'];
chordFilePath=['.' featDir '/' chordFileName];


%copying the wav file under a temporary name (to execute Ueda's system)
[Y,FS,NBITS]=wavread(inputWAV);
wavwrite(Y,FS,NBITS,'./tempWavFile.wav')

%run chord extraction algorithm by Ueda
%the 'songlist.txt' file must contain '/tempWavFile.wav\n' to be valid
!./doChordID.sh "./songlist.txt" "./scratch/" "./rec/"

[stimes,etimes,labelsChord, junk1, labelsKey, junk2]=textread('./scratch/_chroma.rec','%f\t%f\t%s\t%f\t%s\t%f');


fid1=fopen(chordFilePath,'w+');
for k=1:length(stimes)
    fprintf(fid1,'%f\t%f\t%s\t%f\t%s\t%f\n',stimes(k),etimes(k),labelsChord{k,:}, junk1(k), labelsKey{k,:}, junk2(k));
end;

fclose(fid1);

cd('..');

return;


function [beatFileName]=callingDownbeatExtractor(inputWAV,featDir)

% CALLINGDOWNBEATEXTRACTOR function which calls downbeat extractor by Davies and converts its output
%                          in appropriate format.Script gathering downbeats and beats into 1 layer.
%                          A number is given to the location of the beat in the bar :
%                          Note that the number of beats per bar may be tuned, and is fixed to 4.
%
% [beatFileName]=callingDownbeatExtractor(inputWAV,featDir)
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs : - inputWAV : path to input audio wav file (includes filename)
%          - featDir  : fpath to the directory in which extract teh
%                       features
%
% Outputs: - beatFileName : name of the beat file which gathers downbeats and beats
%                           in one layer
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

cd('./downbeat');


%beatFile name :
indSlash=strfind(inputWAV,'/');%filename localization in the path (after the last "/") :
beatFileName=[inputWAV(indSlash(end)+1:end-4) '_BEATS.TXT'];
beatFilePath=[featDir '/' beatFileName];

%number of beats per bar
n=4;
%downbeat matching tolerance
tolerance=0.001;

%Beat and Downbeat extraction (calling Matthiew Davies' script)
[beats,downbeats,df] = main_fn(inputWAV);

%Gathering Beats and Downbeats in one single layer : a number (beat label) is associated
%to each position of a bit in a bar (max 4 beats for one bar).
nbBeats=length(beats);
labels=zeros(1,nbBeats);
%number associated to the position of each beat according to AIST beat
%annotation of RWC POP
numPos=[384,48,96,144];
ind=1; %index of the beat in the current bar (ind=1 in the case of a downbeat)

indexDwnB=0;
for t=1:nbBeats

    %l=length(find(beats(t)==downbeats));

    delta=min(abs(beats(t)-downbeats));


    if(delta<tolerance)
        ind=1;
        labels(t)=numPos(ind);
        indexDwnB=indexDwnB+1;
    else
        %if(ind<=n) %here, max n beats per bar, other beats are labelled "0".
        ind=ind+1;
        ind1=ind-floor((ind-1)/n)*n;%if ind=n+1,2n+1,3n+1... we consider that a new bar of n beats begins
        labels(t)=numPos(ind1);
        %end;
    end;

end;

cd('..');

%Saving this annotation in a text file

fid = fopen(beatFilePath,'w');
for t=1:nbBeats
    fprintf(fid,'%.7f\t%.7f\t%d\n',beats(t)*100,beats(t)*100,labels(t));
end;
fclose(fid);

return;


function [indOcc]=findMatStr(strVect,str)
% FINDMATSTR function searching for a string in a vector of strings
%
% [indOcc]=findMatStr(strVect,str)
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs : - strVect = vector of strings (ith row = ith string)
%          - str     = string
%
% Outputs: - indOcc  = indexes of occurrences of 'str' in 'strVect' (1st
%                      column related to line number, 2nd column related to column number)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[nbStr,lenStr]=size(strVect);
%indOcc=zeros(nbStr,2);
indOcc=[];
for i=1:nbStr
    
    ind=strfind(strVect(i,:),str);
    if(~isempty(ind))
       indOcc=[indOcc; i ind];
    end;
    
end;

return;


function [matSim]=matSimSymb(seq)

% MATSIMSYMB function calculating the similarity matrix of a sequence, with
%            binary similarity measure.
%
% [matSim]=matSimSymb(seq)
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input : - seq = sequence of numbers (1 dimention)
%
% Output: - matSim   = similarity matrix
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

l=length(seq);

matSim=zeros(l,l);

for i=1:l
    for j=1:l
    
        
        if(seq(i)==seq(j))
        
            matSim(i,j)=1;
        else
            matSim(i,j)=0;
        end;
        
    end;
end;
return;



function [chordSequence,keySequence,nbChords,nbKeys]=readChordKeyEstim(chordKeyFile,snapTimes)
% READCHORDKEYESTIM function reading the chord and key annotations from Ueda's system
%
% [chordSequence,keySequence,nbChords,nbKeys]=readChordKeyEstim(chordKeyFile,snapTimes)
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs : - chordKeyFile =  path to .rec file = chord estimations by
%                            Ueda's system.
%          - snapTimes    =  snap occurrences, in seconds
%
% Outputs: - chordSequence= sequence of estimted Chords, mapped with
%                           integers, at the snap scale
%          - keySequence  = sequence of estimted Keys, mapped with
%                           integers, at the snap scale
%          - nbChords     = length of the alphabet of chords
%          - nbKeys       = length of the alphabet of keys 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%chordKeyFile='/Users/gsargent/Work/Versamus/matlab/features/ExtractedChordsFromRWCPopular/11.rec';
%snapTimes=beats;

fid = fopen(chordKeyFile);
        %reading the times and pitches
        C = textscan(fid,'%f\t%f\t%s\t%s\t%s\t%s\n'); %with 1 ground truth
        
fclose(fid);

%output times in seconds (in text file the times are in milliseconds) :
chordTimes00=C{:,1}*10^(-7);
chordTimes01=C{:,2}*10^(-7);
chordLabels=C{:,5};%format : key_chord

nbSmpl=length(chordLabels);

chordLab=zeros(nbSmpl,1);
keyLab=zeros(nbSmpl,1);


%mapping table : letters -> numbers
mapTabChords=['A     ';'A:min ';'A:7   ';'A:aug ';'A:dim ';'A#    ';'A#:min';'A#:7  ';'A#:aug';'A#:dim';'B     ';'B:min ';'B:7   ';'B:aug ';'B:dim ';'B#    ';'B#:min';'B#:7  ';'B#:aug';'B#:dim';'C     ';'C:min ';'C:7   ';'C:aug ';'C:dim ';'C#    ';'C#:min';'C#:7  ';'C#:aug';'C#:dim';'D     ';'D:min ';'D:7   ';'D:aug ';'D:dim ';'D#    ';'D#:min';'D#:7  ';'D#:aug';'D#:dim';'E     ';'E:min ';'E:7   ';'E:aug ';'E:dim ';'E#    ';'E#:min';'E#:7  ';'E#:aug';'E#:dim';'F     ';'F:min ';'F:7   ';'F:aug ';'F:dim ';'F#    ';'F#:min';'F#:7  ';'F#:aug';'F#:dim';'G     ';'G:min ';'G:7   ';'G:aug ';'G:dim ';'G#    ';'G#:min';'G#:7  ';'G#:aug';'G#:dim';'N     '];
mapTabKey=['A  ';'B  ';'C  ';'D  ';'E  ';'F  ';'G  ';'Am ';'Bm ';'Cm ';'Dm ';'Em ';'Fm ';'Gm ';'A# ';'B# ';'C# ';'D# ';'E# ';'F# ';'G# ';'A#m';'B#m';'C#m';'D#m';'E#m';'F#m';'G#m';'N  '];

nbChords=length(mapTabChords);
nbKeys=length(mapTabKey);

%splitting the chord and key labels, separated by a "_", and mapping :
for i=1:nbSmpl
    if(chordLabels{i,:}~='N')
        indUnderscore=findstr(chordLabels{i,:},'_');
        indChord=findMatStr(mapTabChords,chordLabels{i,1}(indUnderscore+1:end));
        chordLab(i)=indChord(1);

        indKey=findMatStr(mapTabKey,chordLabels{i,1}(1:indUnderscore-1));
        keyLab(i)=indKey(1);
        
    else
        %cas o? il n'y a pas d'accord d?tect? -> label 'N'
        chordLab(i)=length(mapTabChords);
        keyLab(i)=length(mapTabKey);
    end;
end;

%METHOD 1 to associate to each beat its closest chord and key symbols in time

nbSnaps=length(snapTimes);
chordSequence=zeros(nbSnaps,1);
keySequence=zeros(nbSnaps,1);

        for t=1:nbSnaps
            %closest rhythmic feature to time t, which stays before time t :
            indClosests=find((chordTimes00-snapTimes(t))<=0);

            %snapTimes(t)-chordTimes00
            %t
            %snapTimes(t)
            %ndClosests
            indClosest=indClosests(end);
            %chordSequence(t)=chordLab(indClosest);
            keySequence(t)=keyLab(indClosest);
        end;

%keyLab
%keySequence


%METHOD 2 to associate to each beat the chord which is played with the
%highest duration within the interval limited by the current snap and its
%following one.

chordTimes00=[chordTimes00;chordTimes01(end)];

%edge effect : adjusting the end time of the last chord with the last snap
%detected if ,needed
if(chordTimes00(end)<snapTimes(end))
	epsilon=0.0001;
	chordTimes00(end)=snapTimes(end)+epsilon;
end;

for t=1:nbSnaps-1

    %recherche des fronti?res contenues dans l'intervalle [snapTimes(t),snapTimes(t+1)]
    ind1=find((chordTimes00-snapTimes(t))>=0);%fronti?res apr?s le t-i?me snap.
    ind2=find((snapTimes(t+1)-chordTimes00(ind1(1):end))>=0);%fronti?res avant le (t+1)-i?me snap.
    indBorders=ind2+ind1(1)-1;
    
    if(length(indBorders)~=0)   %cas o? des fronti?res entre accords tombent entre 2 snaps

        %dur?es relatives aux accords pr?sents dans l'intervalle de temps
        %consid?r? :
        timeBorders=[snapTimes(t);chordTimes00(indBorders);snapTimes(t+1)];

        durations=timeBorders(2:end)-timeBorders(1:end-1);

        %num?ro des accords associ?s aux dur?es :
        if(indBorders(1)==1)
            chL=chordLab([indBorders(1);indBorders]);
        else
            chL=chordLab([indBorders(1)-1;indBorders]);
        end;
        
        %choix de l'accord de plus grande dur?e pour repr?senter le snap(t) :
        [vmax,imax]=max(durations);
[durations,chL];

        chordSequence(t)=chL(imax);
        
    else    %cas o? il n'y a pas de fronti?re entre accords entre 2 snaps, consid?rer la fronti?re pass?e la plus proche et lui assigner le nom de l'accord associ?.
        ind0=find((chordTimes00-snapTimes(t))<=0);
        chordSequence(t)=chordLab(ind0(end));
        %chordLab
        %chordLab(ind0(end))
        %t
        %ind0
        %chordLab(ind0(end))
        %pause;
    end;
end;

chordSequence(nbSnaps)=chordLab(ind0(end));

return;


function [mapSeq,mapTab]=referAndMap(seq)
% REFERANDMAP function referencing the integers which are present in the input sequence
%             and mapping them to the interval [1:nbOfUsefulClusters]
%
% [mapSeq,mapTab]=referAndMap(seq)
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs : - seq = sequence of integers (class numbers)
%
% Outputs: - mapSeq = mapped version of the input sequence
%          - mapTab = mapping table
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

mapSeq=zeros(size(seq));

classMax=max(seq);%highest class number

mapTab=[];
k=0;

for i=1:classMax
    ind=find(seq==i);
    if(length(ind)~=0)
        mapTab=[mapTab i];
        k=k+1;
        mapSeq(ind)=k;
    end;
end;

return;


function [cost]=regCost(k,tau,nType,alpha)
% REGCOST regularity cost functions
%
% [cost]=regCost(k,tau,nType,alpha)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs : - k      = input segment size (in snaps)
%          - tau    = structural pulse period (in snaps)
%          - nType  = choice of cost function type nType=1:3
%          - alpha  = convexity factor(power of the third cost function)    
%
% Outputs: - cost   = cost of k by selected regularity cost function
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch(nType)
    case 1
    cost=(tau(1)/k)-2+(k/tau(1));

    case 2
    gamma=k/tau(1);
    cost=gamma*log(gamma)-log(gamma)/gamma;    

    case 3
        if(k<=tau(1))
            cost=(1-k/tau(1))^alpha;
        else
            cost=((k-tau(1))/tau(1))^alpha;
        end;

end;
return;
